<?php
/**
 * Created by PhpStorm.
 * User: Noa
 * Date: 9/11/2016
 * Time: 1:29 PM
 */

namespace Fr\RetailMeNot\Cloud\StateMachine;

abstract class Element {

    /**
     * State id use to identify it into graph
     * @var string
     */
    private $id = null;

    /**
     * Callback name
     * @var string
     */
    private $callback = '';

    /**
     * Callback parameters
     * @var array
     */
    private $callbackParameters = array();

    /**
     * Store state machine variables
     * @var StateMachine
     */
    private $stateMachine = null;

    /**
     * State constructor.
     * @param string $name
     * @param string $callbackName
     * @param array $callbackParameters
     */
    public function __construct($name, $callbackName = '', array $callbackParameters = array()) {

        $this->id = strval($name);
        $this->callback = $callbackName;
        $this->callbackParameters = $callbackParameters;

    }

    /**
     * Return the state name
     * @return string
     */
    public function getId() {
        return $this->id;
    }

    /**
     * Set a callback to state
     * @param $name Callback name
     * @param array $parameters Callback name parameters (these parameters must exist into state machine machine)
     */
    public function setCallback($name, $parameters = array()) {

        $this->callback = $name;
        $this->callbackParameters = $parameters;
    }

    /**
     * Set callback parameters
     * @param StateMachine $stateMachine Reference to state machine
     */
    public function setStateMachine($stateMachine) {

        $this->stateMachine = $stateMachine;
    }

    /**
     * Run the callback with parameters if given
     * @return mixed
     */
    public function run() {

        if (is_null($this->callback)) {
            return null;
        }

        if (is_null($this->stateMachine)) {
            throw new StateMachineException(StateMachineException::ENVIRONMENT_NOT_DEFINED);
        }

        $parameters = array();
        $environment = $this->stateMachine->getEnvironment();
        //Apply parameters from environment
        foreach($this->callbackParameters as $parameter) {
            if (!array_key_exists($parameter, $environment)) {
                throw new StateMachineException(StateMachineException::ENVIRONMENTAL_PROPERTY_NOT_FOUND);
            }
            $parameters[] = $environment[$parameter];
        }

        $callbackSplitted = explode('::', $this->callback);

        if (sizeof($callbackSplitted) == 2) {

            if(method_exists($callbackSplitted[0], $callbackSplitted[1])) {
                return forward_static_call($callbackSplitted, $this->callbackParameters);
            }

        } else {

            if (function_exists($this->callback)) {
                return call_user_func_array($this->callback, $parameters);
            }

        }
    }

}
