<?php
/**
 * Created by PhpStorm.
 * User: Noa
 * Date: 9/11/2016
 * Time: 12:54 PM
 */

namespace Fr\RetailMeNot\Cloud\StateMachine;

class StateMachine {

    /**
     * @var array<State>
     */
    private $states = array();

    /**
     * @var array<Transition>
     */
    private $transitions = array();

    /**
     * @var array
     */
    private $properties = array();

    /**
     * Current step
     * @var State
     */
    private $currentState =  null;

    /**
     * End state
     * @var State $endState
     */
    private $endState = null;

    /**
     * StateMachine constructor.
     */
    public function __construct() {

    }

    public function setCurretState(State $currentState) {
        $this->currentState = $currentState;
    }

    public function setEndState(State $endState) {
        $this->endState = $endState;
    }

    public function addStep(State $previous, State $next = null, Transition $through = null) {

        // Register steps and transition between them
        $this->addState($previous);
        $this->addState($next);
        $this->addTransition($through);

        // Apply transition to $previous step
        $previous->addOutputTransition($through, $next);
    }

    public function addState(State $state) {

        if (in_array($state->getId(), $this->states)) {
            return;
        }

        $state->setStateMachine($this);

        $this->states[$state->getId()] = array(
            'state' => $state,
        );

    }

    public function addTransition(Transition $transition) {

        if (in_array($transition->getId(), $this->transitions)) {
            return;
        }

        $transition->setStateMachine($this);

        $this->transitions[$transition->getId()] = $transition;

    }

    public function setProperty($propertyName, $property) {

         //if(substr( $propertyName, 0, 3) == "#--")
         $this->properties[$propertyName] = $property;

    }

    /**
     * @param $name
     * @return State
     */
    protected function getState($name) {

        if (array_key_exists($name, $this->states)) {
            return $this->states[$name]['state'];
        }

        return null;

    }


    /**
     * @param $name
     * @return State
     */
    protected function getTransition($name) {

        if (array_key_exists($name, $this->transitions)) {
            return $this->transitions[$name];
        }

        return null;

    }

    public function getEnvironment() {

        return $this->properties;
    }

    public function start() {

        if (is_null($this->currentState)) {
            throw new StateMachineException(StateMachineException::CURRENT_STEP_NOT_DEFINED);
        }

        $this->currentState->run();

    }

    public function next() {

        if (is_null($this->currentState)) {
            throw new StateMachineException(StateMachineException::CURRENT_STEP_NOT_DEFINED);
        }

        $newState = null;

        /**
         * @var Transition $transition
         */
        foreach($this->currentState->getOutputTransitions() as $transition) {

            if ($transition->run()) {
                $newState = $this->currentState->getnextState($transition->getId());
                break;
            }

        }

        if ($newState && $this->currentState->getId() != $newState) {
            $newState->run();
            $this->currentState = $newState;
        }

        if (!is_null($this->endState) && $this->endState->getId() == $this->currentState->getId()) {
            return false;
        }

        return true;

    }

}
